'''SOAP exceptions and faults

Copyright 2007-2011 AG Projects
'''

__all__ = [
    'SOAPError', 'Error',
    'InvalidTypeError', 'InvalidValueError', 'MissingInputError',
    'InternalError', 'StorageError', 'SipProxyError',

    'NotImplementedError',
    'AuthenticationError', 'AuthorizationError', 'ImpersonationError',
    'SipCreditError', 'SipAliasCreditError', 'SipAdvancedCreditError',
    'EnumCreditError',

    'SipDomainNotFoundError', 'SipDomainExistsError',
    'TrustedPeerNotFoundError', 'TrustedPeerExistsError',
    'SipUserNotFoundError',   'SipUserExistsError',
    'SipAliasNotFoundError',  'SipAliasExistsError',
    'AlreadyInSipGroupError', 'NotInSipGroupError',

    'InvalidPrepaidCardError', 'NotPrepaidAccountError',

    'PhonebookEntryNotFoundError', 'PhonebookEntryExistsError',

    'VoicemailNotFoundError', 'VoicemailExistsError',

    'EnumRangeNotFoundError',  'EnumRangeExistsError',
    'EnumNumberNotFoundError', 'EnumNumberExistsError',
    'EnumNumberRangeError', 
    'EnumMappingNotFoundError', 'TooManyEnumMappingsError',
    
    'RatingEngineConnectionError', 'RatingEntityNotFoundError', 
    'RatingProfilesNotFoundError',
    
    'CustomerNotFoundError', 'CustomerExistsError',

    'XCAPError'
    ]

## First we override the faultType from SOAP with our own

import SOAPpy

## Save old faultType which we will overwrite
SOAPpy.Types._faultType = SOAPpy.Types.faultType ## save old

class faultType(SOAPpy.Types._faultType):
    def __init__(self, faultcode='', faultstring='', detail=None):
        SOAPpy.Types._faultType.__init__(self, faultcode, faultstring, detail)
        self._setDetail(detail)

    def _setDetail(self, detail=None):
        if not isinstance(self, Error):
            import sys
            info = sys.exc_info()
            try:
                faultdetail = {'exception': {'name': 'InternalError',
                                             'errorcode': InternalError.errorcode}}
                if info != (None, None, None):
                    faultdetail['exception']['errorstring'] = "%s %s" % info[0:2]
                elif detail is str:
                    faultdetail['exception']['errorstring'] = detail
                else:
                    faultdetail['exception']['errorstring'] = self.faultstring
                self.detail = faultdetail
            finally:
                del(info)

## Now overwrite the SOAP faultType so we can control how we
## report uncaptured errors
SOAPpy.faultType        = faultType
SOAPpy.Types.faultType  = faultType
SOAPpy.Server.faultType = faultType
SOAPpy.Client.faultType = faultType

from SOAPpy import NS

class Error(faultType):
    faultyParty = 'Server'
    faultstring = 'Could not process request'
    errorcode = 1
    name = None

    def __init__(self, error):
        env = NS.ENV_T
        exception = self.name or self.__class__.__name__
        faultcode = '%s:%s' % (env, self.faultyParty)
        faultstring = self.faultstring
        faultdetail = {'exception': {'name': exception,
                                     'errorcode': self.errorcode, 
                                     'errorstring': str(error)}}
        faultType.__init__(self, faultcode, faultstring, faultdetail)

SOAPError = Error

class InvalidTypeError(Error):
    faultyParty = 'Client'
    faultstring = 'Invalid input data type'
    errorcode = 100

class InvalidValueError(Error):
    faultyParty = 'Client'
    faultstring = 'Invalid input data value'
    errorcode = 101

class MissingInputError(Error):
    faultyParty = 'Client'
    faultstring = 'Missing required input'
    errorcode = 102

class InternalError(Error):
    faultyParty = 'Server'
    faultstring = 'Server internal error'
    errorcode = 200

class StorageError(Error):
    faultyParty = 'Server'
    faultstring = 'Storage error'
    errorcode = 201

class SipProxyError(Error):
    faultyParty = 'Server'
    faultstring = 'Error accessing the SER control fifo'
    errorcode = 202

class NotImplementedError(Error):
    faultyParty = 'Server'
    faultstring = 'Method not implemented'
    errorcode = 900

class AuthenticationError(Error):
    faultyParty = 'Client'
    faultstring = 'Invalid authentication'
    errorcode = 901

class AuthorizationError(Error):
    faultyParty = 'Client'
    faultstring = 'Client not authorized to perform operation'
    errorcode = 902

class ImpersonationError(Error):
    faultyParty = 'Client'
    faultstring = 'Cannot impersonate account'
    errorcode = 903

class SipCreditError(Error):
    faultyParty = 'Client'
    faultstring = 'Not enough SIP credit to perform operation'
    errorcode = 950

class SipAliasCreditError(Error):
    faultyParty = 'Client'
    faultstring = 'Not enough SIP alias credit to perform operation'
    errorcode = 951

class SipAdvancedCreditError(Error):
    faultyParty = 'Client'
    faultstring = 'Not enough advanced SIP credit to perform operation'
    errorcode = 952

class EnumCreditError(Error):
    faultyParty = 'Client'
    faultstring = 'Not enough ENUM credit to perform operation'
    errorcode = 953

##
## Sip
##
class SipDomainNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'Domain not found'
    errorcode = 1000

class SipDomainExistsError(Error):
    faultyParty = 'Server'
    faultstring = 'Domain already exists'
    errorcode = 1001

class TrustedPeerNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'Trusted peer not found'
    errorcode = 1005

class TrustedPeerExistsError(Error):
    faultyParty = 'Server'
    faultstring = 'Trusted peer already exists'
    errorcode = 1006

class SipUserNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'User not found'
    errorcode = 1010

class SipUserExistsError(Error):
    faultyParty = 'Server'
    faultstring = 'User already exists'
    errorcode = 1011

class SipAliasNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'Alias not found'
    errorcode = 1020

class SipAliasExistsError(Error):
    faultyParty = 'Server'
    faultstring = 'Alias already exists'
    errorcode = 1021

class AlreadyInSipGroupError(Error):
    faultyParty = 'Server'
    faultstring = 'User already in group'
    errorcode = 1030

class NotInSipGroupError(Error):
    faultyParty = 'Server'
    faultstring = 'User not in group'
    errorcode = 1031

class PhonebookEntryNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'Phonebook entry not found'
    errorcode = 1040

class PhonebookEntryExistsError(Error):
    faultyParty = 'Server'
    faultstring = 'Phonebook entry already exists'
    errorcode = 1041

class InvalidPrepaidCardError(Error):
    faultyParty = 'Client'
    faultstring = 'Invalid prepaid account'
    errorcode = 1050
        
class NotPrepaidAccountError(Error):
    faultyParty = 'Client'
    faultstring = 'Not a prepaid account'
    errorcode = 1051

##
## Voicemail
##
class VoicemailNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'User has no voicemail account'
    errorcode = 2000

class VoicemailExistsError(Error):
    faultyParty = 'Server'
    faultstring = 'User already has voicemail account'
    errorcode = 2001

##
## Enum
##
class EnumRangeNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'ENUM range not found'
    errorcode = 3000

class EnumRangeExistsError(Error):
    faultyParty = 'Server'
    faultstring = 'ENUM range already exists'
    errorcode = 3001

class EnumNumberNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'ENUM number not found'
    errorcode = 3002

class EnumNumberExistsError(Error):
    faultyParty = 'Server'
    faultstring = 'ENUM number already exists'
    errorcode = 3003

class EnumNumberRangeError(Error):
    faultyParty = 'Server'
    faultstring = 'Cannot find an appropriate ENUM range for the request'
    errorcode = 3004

class EnumMappingNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'ENUM mapping not found'
    errorcode = 3005

class TooManyEnumMappingsError(Error):
    faultyParty = 'Server'
    faultstring = 'Too many ENUM mappings for this number'
    errorcode = 3006

##
## Rating
##
class RatingEngineConnectionError(Error):
    faultyParty = 'Server'
    faultstring = 'Failed to connect to the Rating Engine'
    errorcode = 4000

class RatingEntityNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'Rating Entity does not exist'
    errorcode = 4001

class RatingProfilesNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'Rating Profiles do not exist'
    errorcode = 4002 

##
## Customer
##
class CustomerNotFoundError(Error):
    faultyParty = 'Server'
    faultstring = 'Customer not found'
    errorcode = 5000

class CustomerExistsError(Error):
    faultyParty = 'Server'
    faultstring = 'Customer already exists'
    errorcode = 5001

##
## XCAP
##
class XCAPError(Error):
    faulyParty = 'Server'
    faultstring = 'XCAP HTTP error'
    errorcode = 6000
