'''SOAP data types described by the WSDL

Copyright 2007-2011 AG Projects
'''

## We import SOAPTweaks to be able to work standalone, not only when included by SOAPEngine.
## Without this the generated SOAP output will be wrong.

from ssp.soap import tweaks
import SOAPpy
import md5, re
from time import time, strptime
from application.configuration import ConfigSection

from ssp.soap.errors import *


class SOAPServerConfig(ConfigSection):
    __cfgfile__ = 'config.ini'
    __section__ = 'SOAP Server'
    namespace = 'urn:AGProjects:SoapSIMPLEProxy'


def isSIPUri(uri):
    '''Check if uri is in the form sip:username@domain or a valid keyword like <voice-mailbox>'''
    keywords = ('<voice-mailbox>',)
    if uri in keywords:
        return True
    #pattern = r'(sip|sips):\+?\w(([\w.-]*?\w)|)@([a-z0-9](([a-z0-9-]*?[a-z0-9])|)\.)+[a-z]{2,}$'
    pattern = r'(sip|sips):\+?\w(([\w.-]*?\w)|)@(([a-z0-9](([a-z0-9-]*?[a-z0-9])|)\.)+[a-z]{2,}|([0-9]{1,3}\.){3}[0-9]{1,3})$'
    return re.match(pattern, uri, re.IGNORECASE|re.UNICODE) is not None

def soapTypeName(cls):
    '''Return SOAP type name for the specified class'''
    if cls is None:                   return 'object'
    elif issubclass(cls, basestring): return 'string'
    elif issubclass(cls, long):       return 'integer'
    elif issubclass(cls, float):      return 'double'
    else:                             return cls.__name__

## Generic type checking classes. Not for use inside a Structure or Array
## They also do not reflect any particular SOAP data type. Used only to
## enforce certain restrictions, because the python SOAP module has
## relaxed type checking and sometimes we need more strict checking.

## _Int and _Integer - simple checking of integers.
class _Int(int):
    def __new__(typ, *args, **kwargs):
        try:
            obj = args[0]
        except IndexError:
            pass
        else:
            if type(obj) is not int:
                objname = int.__new__(typ).__class__.__name__
                raise InvalidTypeError, "%s should be an int" % objname
        return int.__new__(typ, *args, **kwargs)

class _Integer(long):
    def __new__(typ, *args, **kwargs):
        try:
            obj = args[0]
        except IndexError:
            pass
        else:
            if type(obj) not in (int, long):
                objname = long.__new__(typ).__class__.__name__
                raise InvalidTypeError, "%s should be an integer" % objname
        return long.__new__(typ, *args, **kwargs)

## a non-empty string
class _String(unicode):
    _allowEmpty = 0
    def __init__(self, obj):
        objname = self.__class__.__name__
        if type(obj) not in (str, unicode):
            raise InvalidTypeError, "%s should be a string" % objname
        if not self._allowEmpty and obj.strip() == '':
            raise InvalidValueError, "%s should not be empty" % objname
        unicode.__init__(self, obj)


##
## SOAP Data Types
##

## This is the object to return from function that have no return value
## This is needed, because not returning anything from a python function,
## is interpreted as None, and the SOAP engine will pack that None value
## as the return value of the function, which we need to avoid
void = SOAPpy.voidType()

## string type. This is here only to have a string with the same name
## as the corresponding SOAP string type. For use with Structure/Array
class string(unicode): pass

## Base class for any SOAP data type that doesn't directly map to a python
## type. Such classes should inherit this class.
class SOAPDataType(object): pass

## Classes that do not directly map into python types. Should inherit
## from the closest matching python type and from SOAPDataType

class nonNegativeInteger(long, SOAPDataType):
    def __new__(typ, *args, **kwargs):
        try:
            obj = args[0]
        except IndexError:
            pass
        else:
            if type(obj) not in (int, long) or obj < 0:
                raise TypeError
        return long.__new__(typ, *args, **kwargs)

## Classes that represent the SOAP simple types. Should inherit
## from the closest matching python type and from SOAPDataType

class IPAddress(string, SOAPDataType):
    constraint = r'^(\d{1,3}\.){3}\d{1,3}$'
    regexp = re.compile(constraint)
    
    def __new__(typ, *args, **kwargs):
        try:
            obj = args[0]
        except IndexError:
            pass
        else:
            if not isinstance (obj, basestring):
                raise TypeError
            if not IPAddress.regexp.match(obj):
                raise ValueError, "IPAddress should match the '%s' constraint." % IPAddress.constraint
        return string.__new__(typ, *args, **kwargs)

## The base classes for array like and dictionary like types: Array and Structure
class Array(list):
    _type = None
    _ns   = SOAPServerConfig.namespace
    def __init__(self, obj):
        objname = self.__class__.__name__
        if not isinstance(obj, (tuple, list)):
            raise InvalidTypeError, "%s should be an array" % objname
        try:
            list.__init__(self, self._type and [self._type(o) for o in obj] or obj)
        except InvalidTypeError, why:
            raise InvalidTypeError, "invalid %s element: %s" % (objname, why)
    def getso(self):
        if issubclass(self._type, (Array, Structure)):
            out = SOAPpy.arrayType([x.SOAPout for x in self])
        else:
            out = SOAPpy.arrayType(list(self)) # not very efficient. arrayType should be modified to accept any list/tuple derivative
        out._typed = True
        out._type = soapTypeName(self._type)
        out._complexType = True
        out._typename = self.__class__.__name__
        out._ns = self._ns
        return out
    SOAPout  = property(getso)
    del getso

class Structure(dict):
    _structure = {}
    _dbmap = {}
    def __init__(self, obj):
        objname = self.__class__.__name__
        if not isinstance(obj, dict):
            raise InvalidTypeError, "%s should be a struct" % objname
        #ignore any element that is not part of this Structure:
        for k in obj.keys():
            if k not in self._structure: del obj[k]
        for (elem, description) in self._structure.items():
            try:
                val = obj[elem]
            except KeyError:
                val = description.get('default', None)
            if val is None:
                if not description['nillable']:
                    raise MissingInputError, "%s in %s cannot be missing" % (elem, objname)
                else:
                    self[elem] = None
                    continue
            enumeration = description.get('enumeration', None)
            if enumeration is not None and val not in enumeration:
                raise InvalidValueError, "Invalid value `%s' for %s.%s. Valid values are: %s" % (val, objname, elem, ', '.join(enumeration))
            _type = description['type']
            if issubclass(_type, Structure):
                self[elem] = _type(val)
            elif issubclass(_type, Array):
                try:
                    self[elem] = _type(val)
                except TypeError:
                    raise InvalidTypeError, "%s should be %s" % (elem, _type.__name__)
            elif issubclass(_type, SOAPDataType):
                try:
                    self[elem] = _type(val)
                except TypeError:
                    raise InvalidTypeError, "%s should be %s" % (elem, _type.__name__)
            else:
                ## make a list of acceptable types for the input data
                if _type is string:
                    _types = (str, unicode)
                elif _type is long:
                    _types = (int, long)
                else:
                    _types = (_type,)
                ## tweak the input value if needed
                if _type is int and type(val) is long:
                    val = int(val)  ## we accept long integers as long as they don't overflow
                elif _type is bool and type(val) is int and val in (0, 1):
                    val = bool(val) # Temporary hack (until SOAPpy returns bool for a boolean)
                ## check that input type is acceptable
                if type(val) not in _types:
                    raise InvalidTypeError, "%s should be %s" % (elem, _type.__name__)
                ## everything that accepts multiple input types (except string) must be encoded as
                ## the declared type. string declared types will retain their type (str or unicode)
                if _type is long:
                    self[elem] = _type(val)
                else:
                    self[elem] = val
        self.__dict__['_ns'] = SOAPServerConfig.namespace
        #self._dbconditions = {} # check if we need to keep this -Dan
        self.__dict__['_dbconditions'] = {} # check if we need to keep this -Dan
    def __setitem__(self, key, value):
        self.__dict__[key] = value
        dict.__setitem__(self, key, value)
    def __delitem__(self, key):
        dict.__delitem__(self, key)
        del self.__dict__[key]
    __setattr__ = __setitem__
    def __delattr__(self, name):
        try:
            del self.__dict__[name]
        except KeyError:
            raise AttributeError, "'%s' object has no attribute '%s'" % (self.__class__.__name__, name)
        else:
            dict.__delitem__(self, name)
    def update(self, other):
        dict.update(self, other)
        for key, value in other.items():
            self.__dict__[key] = value
    def getso(self):
        out = {}
        for (elem, description) in self._structure.items():
            _class = description['type']
            value = self[elem]
            if value is None:
                out[elem] = value
            elif issubclass(_class, (Array, Structure)):
                out[elem] = value.SOAPout
            elif _class is string: ## `string' is the only type not encoded as declared (it's either str or unicode)
                out[elem] = _class(value)
            elif _class is bool: # temporary workaround (SOAPpy.SOAPBuilder should be fixed to correctly marshall bool values) -Dan
                out[elem] = SOAPpy.booleanType(value)
            else:
                out[elem] = value
        out = SOAPpy.structType(out)
        out._ns = self._ns
        out._typename = self.__class__.__name__
        return out
    SOAPout = property(getso)
    del getso
    def getinfo(self):
        '''Return a dictionary with all properties that are present'''
        info = {}
        for prop in self.__dict__:
            if prop.startswith('_'):
                continue
            val = self[prop]
            if val is not None:
                info[prop] = val
        return info
    def dbinfo(self):
        info = {}
        map = self._dbmap
        for prop in map:
            try:
                val = eval('self.%s' % prop) ## we use this so that 'self.id.username' works, unlike with self[prop]
            except AttributeError:
                pass
            else:
                if val is not None:
                    info[map[prop]] = val
        return info
    # keep this? -Dan
    def dbconditions(self):
        conditions = {}
        map = self._dbmap
        for prop in map:
            val = self._dbconditions.get(prop)
            if val is not None:
                conditions[map[prop]] = val
        return conditions
    def checkNonEmpty(self, properties):
        '''Raise error if the specified properties are present but empty'''
        objname = self.__class__.__name__
        for prop in properties:
            val = self[prop]
            if (val is not None) and (not val) and (val != 0):
                raise MissingInputError, "%s property of %s cannot be empty" % (prop, objname)
    def checkRequired(self, properties):
        '''Raise error if the specified properties are missing or empty'''
        objname = self.__class__.__name__
        for prop in properties:
            val = self[prop]
            if (not val) and (val != 0):
                raise MissingInputError, "%s property of %s cannot be missing or empty" % (prop, objname)
    def checkNotAllowed(self, properties):
        '''Raise error if the specified properties are present'''
        objname = self.__class__.__name__
        for prop in properties:
            if self[prop] is not None:
                raise InvalidValueError, "%s property of %s cannot be specified" % (prop, objname)

##
## The SOAP data types as described by the WSDL
##

class StringArray(Array):
    _type = string

class IntArray(Array):
    _type = int

class IntegerArray(Array):
    _type = long

class SipId(Structure):
    _structure = {'username': {'type': string, 'nillable': False},
                  'domain':   {'type': string, 'nillable': False}}    

    def __init__(self, obj):
        Structure.__init__(self, obj)
        if not self.username or not self.domain:
            raise InvalidValueError("Username and domain cannot be empty.")

class SipIdArray(Array):
    _type = SipId

class Watcher(Structure):
    _structure = {'id':      {'type': string, 'nillable': False},
                  'status':  {'type': string, 'nillable': False},
                  'online':  {'type': bool,   'nillable': False}}

class WatcherArray(Array):
    _type = Watcher

class PresenceInformation(Structure):
    _structure = {'activity': {'type': string, 'nillable': False},
                  'note':     {'type': string, 'nillable': True}}

class PresencePolicy(Structure):
    _structure = {'allow':      {'type': StringArray, 'nillable': True},
                  'deny' :      {'type': StringArray, 'nillable': True},
                  'confirm' :   {'type': StringArray, 'nillable': True}}
